/* SPDX-License-Identifier: (GPL-2.0+ OR MIT) */
/*
 * Copyright (c) 2020 BayLibre, SAS
 * Author: Neil Armstrong <narmstrong@baylibre.com>
 */

#ifndef _DT_BINDINGS_MESON_AXG_POWER_H
#define _DT_BINDINGS_MESON_AXG_POWER_H

#define PWRC_AXG_VPU_ID			0
#define PWRC_AXG_ETHERNET_MEM_ID	1
#define PWRC_AXG_AUDIO_ID		2

#endif
