/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */

#ifndef __DT_BINDINGS_SOC_FSL_QE_TSA_H
#define __DT_BINDINGS_SOC_FSL_QE_TSA_H

#define FSL_QE_TSA_NU		0
#define FSL_QE_TSA_UCC1		1
#define FSL_QE_TSA_UCC2		2
#define FSL_QE_TSA_UCC3		3
#define FSL_QE_TSA_UCC4		4
#define FSL_QE_TSA_UCC5		5

#endif
