/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef __DT_BINDINGS_CLOCK_BCM6368_H
#define __DT_BINDINGS_CLOCK_BCM6368_H

#define BCM6368_CLK_VDSL_QPROC		2
#define BCM6368_CLK_VDSL_AFE		3
#define BCM6368_CLK_VDSL_BONDING	4
#define BCM6368_CLK_VDSL		5
#define BCM6368_CLK_PHYMIPS		6
#define BCM6368_CLK_SWPKT_USB		7
#define BCM6368_CLK_SWPKT_SAR		8
#define BCM6368_CLK_SPI			9
#define BCM6368_CLK_USBD		10
#define BCM6368_CLK_SAR			11
#define BCM6368_CLK_ROBOSW		12
#define BCM6368_CLK_UTOPIA		13
#define BCM6368_CLK_PCM			14
#define BCM6368_CLK_USBH		15
#define BCM6368_CLK_DIS_GLESS		16
#define BCM6368_CLK_NAND		17
#define BCM6368_CLK_IPSEC		18

#endif /* __DT_BINDINGS_CLOCK_BCM6368_H */
