/* SPDX-License-Identifier: GPL-2.0 */
/*
 * CS42L43 CODEC driver external data
 *
 * Copyright (C) 2022-2023 Cirrus Logic, Inc. and
 *                         Cirrus Logic International Semiconductor Ltd.
 */

#ifndef CS42L43_ASOC_EXT_H
#define CS42L43_ASOC_EXT_H

#define CS42L43_SYSCLK		0

#define CS42L43_SYSCLK_MCLK	0
#define CS42L43_SYSCLK_SDW	1

#endif /* CS42L43_ASOC_EXT_H */
