/* SPDX-License-Identifier: GPL-2.0+ */

#ifndef __DT_BINDINGS_BMIPS_BCM6328_PM_H
#define __DT_BINDINGS_BMIPS_BCM6328_PM_H

#define BCM6328_POWER_DOMAIN_ADSL2_MIPS	0
#define BCM6328_POWER_DOMAIN_ADSL2_PHY	1
#define BCM6328_POWER_DOMAIN_ADSL2_AFE	2
#define BCM6328_POWER_DOMAIN_SAR	3
#define BCM6328_POWER_DOMAIN_PCM	4
#define BCM6328_POWER_DOMAIN_USBD	5
#define BCM6328_POWER_DOMAIN_USBH	6
#define BCM6328_POWER_DOMAIN_PCIE	7
#define BCM6328_POWER_DOMAIN_ROBOSW	8
#define BCM6328_POWER_DOMAIN_EPHY	9

#endif /* __DT_BINDINGS_BMIPS_BCM6328_PM_H */
