/* SPDX-License-Identifier: GPL-2.0 */
#ifndef _DT_BINDINGS_POWER_MT6765_POWER_H
#define _DT_BINDINGS_POWER_MT6765_POWER_H

#define MT6765_POWER_DOMAIN_CONN		0
#define MT6765_POWER_DOMAIN_MM			1
#define MT6765_POWER_DOMAIN_MFG_ASYNC		2
#define MT6765_POWER_DOMAIN_ISP			3
#define MT6765_POWER_DOMAIN_MFG			4
#define MT6765_POWER_DOMAIN_MFG_CORE0		5
#define MT6765_POWER_DOMAIN_CAM			6
#define MT6765_POWER_DOMAIN_VCODEC		7

#endif /* _DT_BINDINGS_POWER_MT6765_POWER_H */
