/* SPDX-License-Identifier: (GPL-2.0-only OR MIT) */
/*
 * Copyright (C) 2025 Amlogic, Inc. All rights reserved
 */
#ifndef _DT_BINDINGS_AMLOGIC_S6_POWER_H
#define _DT_BINDINGS_AMLOGIC_S6_POWER_H

#define PWRC_S6_DSPA_ID				0
#define PWRC_S6_DOS_HEVC_ID			1
#define PWRC_S6_DOS_VDEC_ID			2
#define PWRC_S6_VPU_HDMI_ID			3
#define PWRC_S6_U2DRD_ID			4
#define PWRC_S6_U3DRD_ID			5
#define PWRC_S6_SD_EMMC_C_ID			6
#define PWRC_S6_GE2D_ID				7
#define PWRC_S6_AMFC_ID				8
#define PWRC_S6_VC9000E_ID			9
#define PWRC_S6_DEWARP_ID			10
#define PWRC_S6_VICP_ID				11
#define PWRC_S6_SD_EMMC_A_ID			12
#define PWRC_S6_SD_EMMC_B_ID			13
#define PWRC_S6_ETH_ID				14
#define PWRC_S6_PCIE_ID				15
#define PWRC_S6_NNA_4T_ID			16
#define PWRC_S6_AUDIO_ID			17
#define PWRC_S6_AUCPU_ID			18
#define PWRC_S6_ADAPT_ID			19

#endif
